#include "SyncScheduler.hpp"
#include "DataInputWindow.hpp"
#include "MainWindow.hpp"
#include "SoftkeyWindow.hpp"
#include "syncscheduler_res.h"
#define     LABEL_SOFTKEY1          ("Finish")
#define     LABEL_SOFTKEY3          ("Select")
#define MENU_TOP_MARGIN    20
#define MENU_SIDE_MARGIN   20
#define MENU_SPACE         15
#define MENU_BOX_WIDTH     30
#define TEXTBOX_REVISING   6

DataInputWindow::DataInputWindow(SFXDateConstRef date) : SFRPlainWindow(SFRApplication::GetInstance(), SyncScheduler::CalculateWindowPosition()), _date(date) static_throws
{
    // Registration of soft key
    SoftkeyWindowPtr softkey;

    SoftkeyWindow::Initialize(this);
    if (static_try()) {
        // Registration of drawing handler
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT,
        HANDLER_BEFORE, HANDLER_FUNCTION(OnRenderContent)));
        if (static_try()) {
            // Registration of key handler
            static_throw(RegisterHandler(SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
            if (static_try()) {
                //static_throw(RegisterHandler(static_cast<SFCEventEnum>(SoftkeyWindow::USEREVT_SOFTKEY), HANDLER_AFTER, HANDLER_FUNCTION(OnSoftkey)));
                if (static_try()) {
                    if ((softkey = SoftkeyWindow::GetInstance()) != null) {
                        static_throw(softkey->Register(this, SoftkeyWindow::SOFTKEY_1, LABEL_SOFTKEY1));
                        if (static_try()) {
                            //static_throw(softkey->Register(this, SoftkeyWindow::SOFTKEY_2, LABEL_SOFTKEY2));
                            if (static_try()) {
                                static_throw(softkey->Register(this, SoftkeyWindow::SOFTKEY_3, LABEL_SOFTKEY3));
                                if (static_try()) {
                                    SetReference(softkey->Bind(this));
                                    _fontHeight = SFXGraphics::GetFontHeight(AEE_FONT_NORMAL);
                                    SFXRectangleConst screenRect(GetContentWorld());
                                    SFXRectangle rect(MENU_SIDE_MARGIN, MENU_TOP_MARGIN,
                                    screenRect.GetRight() - MENU_SIDE_MARGIN, _fontHeight);
                                    // Make label
                                    _labelDate = ::new SFRLabelControl(this, rect, date.Format("YYYY/MM/DD"));
                                    rect.AddY(_fontHeight + MENU_SPACE);
                                    rect.SetSize(MENU_BOX_WIDTH, _fontHeight + TEXTBOX_REVISING);
                                    // Make text control for inputting hours
                                    _textHour = ::new SFREditboxControl(this, rect, SFXWideString::EmptyInstance());
                                    _textHour->SetInputMode(AEE_TM_NUMBERS);
                                    _textHour->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));
                                    rect.AddX(MENU_BOX_WIDTH);
                                    // Make label
                                    _labelHour = ::new SFRLabelControl(this, rect, "H");
                                    rect.AddX(rect.GetWidth());
                                    // Make Text control for inputting minutes
                                    _textMinute = ::new SFREditboxControl(this, rect, SFXWideString::EmptyInstance());
                                    _textMinute->SetInputMode(AEE_TM_NUMBERS);
                                    _textMinute->RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));
                                    rect.AddX(MENU_BOX_WIDTH);
                                    // Make label
                                    _labelMinute = ::new SFRLabelControl(this, rect, "M");
                                    rect.SetX(MENU_SIDE_MARGIN);
                                    rect.AddY(_fontHeight + MENU_SPACE + TEXTBOX_REVISING);
                                    rect.SetSize(screenRect.GetRight() - 2 * MENU_SIDE_MARGIN,
                                    screenRect.GetBottom() - 4 * MENU_SPACE
                                    - 3 * _fontHeight - MENU_TOP_MARGIN);
                                    // Make Text control for inputting content
                                    _textContent = ::new SFREditboxControl(this, rect, SFXWideString::EmptyInstance());
                                    _textContent-> RegisterHandler(SREVT_CONTROL, HANDLER_BEFORE, HANDLER_FUNCTION(OnEditboxHandler));
                                    // Make OK button
                                    SInt16 MENU_BUTTON_WIDTH1 = SFXGraphics::MeasureText(AEE_FONT_NORMAL, "OK") +10;
                                    rect.Set(MENU_SIDE_MARGIN, screenRect.GetBottom() - MENU_SPACE - _fontHeight,
                                    MENU_BUTTON_WIDTH1, _fontHeight+4);
                                    _buttonOk = ::new SFRButtonControl(this, rect, "OK");
                                    // Make cancel button
                                    SInt16 MENU_BUTTON_WIDTH2 = SFXGraphics::MeasureText(AEE_FONT_NORMAL, "Cancel") +10;
                                    rect.Set(MENU_SIDE_MARGIN, screenRect.GetBottom() - MENU_SPACE - _fontHeight,
                                    MENU_BUTTON_WIDTH2, _fontHeight+4);
                                    rect.AddX(MENU_SPACE + MENU_BUTTON_WIDTH1);
                                    _buttonCancel = ::new SFRButtonControl(this, rect, "CANCEL");
                                    if (static_try()) {
                                        // The select key is processed by the default handler. (For the text control)
                                        static_throw(RegisterHandler(SFEVT_KEY, AVK_SELECT,
                                        HANDLER_AFTER, HANDLER_FUNCTION(SelectHandler)));
                                        if (static_try()) {
                                            if (_textHour != null && _textMinute != null && _textContent != null) {
                                                // Setting the maximum character number
                                                _textHour->SetMaxSize(2);
                                                _textMinute->SetMaxSize(2);
                                                _textContent->SetMaxSize(255);
                                                // Focus is set. 
                                                _textHour->SetStatusFocus(true);
                                                if (static_try()) {
                                                    if (_buttonOk != null) {
                                                        // Registration of button handler
                                                        static_throw(_buttonOk->RegisterHandler(SREVT_CONTROL, 
                                                        HANDLER_BEFORE,HANDLER_FUNCTION(OnOkButtonControl)));
                                                        if (static_try()) {
                                                            if (_buttonCancel != null) {
                                                                // Registration of button handler
                                                                static_throw(_buttonCancel->RegisterHandler(
                                                                SREVT_CONTROL, HANDLER_BEFORE,
                                                                HANDLER_FUNCTION(OnCancelButtonControl)));
                                                            }
                                                            else {
                                                                static_throw(SFERR_NO_MEMORY);
                                                            }
                                                        }
                                                    }
                                                    else {
                                                        static_throw(SFERR_NO_MEMORY);
                                                    }
                                                }
                                            }
                                            else {
                                                static_throw(SFERR_NO_MEMORY);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return;
}


// Destructor
DataInputWindow::~DataInputWindow(Void)
{
    SoftkeyWindow::Terminate(this);
    return;
}


// Drawing handler
HANDLER_IMPLEMENT_VOIDRENDER(DataInputWindow, OnRenderContent, graphics)
{
    SFRPlainWindow::ContentHandler(graphics);
    return;
}


// Key handler
HANDLER_IMPLEMENT_BOOLEVENT(DataInputWindow, OnKey, event)
{
    Bool result(false);

    switch (event.GetP16()) {
    case AVK_CLR:			
        // Clear key is pressed
        if (_textHour->GetStatusTarget() || _textMinute->GetStatusTarget() || _textContent->GetStatusTarget()) {
            _textHour->SetStatusTarget(false);
            _textMinute->SetStatusTarget(false);
            _textContent->SetStatusTarget(false);
            FocusNext();
            result = true;
        }
        else {
            // Close window
            result = Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
        }
        break;
    case AVK_UP:
        FocusUp();			 
        result = true;
        break;
    case AVK_LEFT:
        FocusLeft();		
        result = true;
        break;
    case AVK_DOWN:
        FocusDown();	
        result = true;
        break;
    case AVK_RIGHT:
        FocusRight();		
        result = true;
        break;
    }
    return result;
}


// Button handler
HANDLER_IMPLEMENT_VOIDCONTROL(DataInputWindow, OnOkButtonControl, result, control)
{
    unused(control);
    unused(result);
    SFXDate start(_date);
    // Convert the string Hour to integer value
    SInt16 hour = static_cast<SInt16>(_textHour->GetText().AsSInt32());
    if (hour < 0 || hour >= 24) {
        // Check Hour
        hour = 0;
    }
    start.SetHour(hour);
    // Convert the string Minute to integer value
    SInt16 minute = static_cast<SInt16>(_textMinute->GetText().AsSInt32());
    if (minute < 0 || minute >= 60) {
        // Check Minute
        minute = 0;
    }
    start.SetMinute(minute);
    // Seconds is assumed to be 0 
    start.SetSecond(0); 
    // Get Content string
    SFXAnsiString summary(static_cast<SFXAnsiString>(_textContent->GetText()));
    // Make data (Processing different depending on kind of window)
    MakeData(start, summary);
    // Get window in background (OneDayWindow)
    SFRResponderPtr oneDayWindow = GetNext();
    // Both the window in the background and the current window are closed. 
    Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
	oneDayWindow->InvalidateContent();
	result = true;
    return;
}


// Button hanlder
HANDLER_IMPLEMENT_VOIDCONTROL(DataInputWindow, OnCancelButtonControl, result, control)
{
    unused(control);
    unused(result);
    // Close current window
    Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE, SRP16_TERMINATE_INVOKE, true));
    return;
}


CreateWindow::CreateWindow(SFXDateConstRef date) : DataInputWindow(date)
{
    return;
}


Void CreateWindow::MakeData(SFXDateConstRef start, SFXAnsiStringConstRef summary)
{
    VCalendarFilePtr vcal = ::new VCalendarFile("1.0", start, start, summary);
    if (vcal != null) {
        SyncScheduler::GetVCalCollection()->Append(vcal);
    }
    return;
}


ModifyWindow::ModifyWindow(SFXDateConstRef date, VCalendarFilePtr vcal)
: DataInputWindow(date), // Call parent constructor
_vcal(vcal) // Data for modifying
{
    SFXDate start = _vcal->GetStartDate();
    // Set initial value of the hour text control
    _textHour->SetText(SFXWideString::Format("%d", start.GetHour()));
    // Set initial value of the minute text control
    _textMinute->SetText(SFXWideString::Format("%02d", start.GetMinute()));
    // Set initial value of the content text control
    _textContent->SetText(_vcal->GetSummary());
    return;
}


Void ModifyWindow::MakeData(SFXDateConstRef start, SFXAnsiStringConstRef summary)
{
    if (_vcal != null) {
        _vcal->Set(start, start, start ,summary);
        SyncScheduler::GetVCalCollection()->Update(_vcal);
    }
    return;
}


// Automatically focus the next edit box when input finished
HANDLER_IMPLEMENT_VOIDCONTROL(DataInputWindow, OnEditboxHandler, result, control)
{
    control->SetStatusTarget(false);
    if (result) {
        next = control->GetNext(TYPE_WILDCARD, ATTRIBUTE_WILDCARD);
        if (next != null) {
            next->SetStatusFocus(true);
        }
    }
    return;  
}

