#ifndef __DATAINPUTWINDOW_HPP
#define __DATAINPUTWINDOW_HPP

#include <SophiaFramework.hpp>

// To display the window, the class inherits the SFRTitleWindow class. 
SFMTYPEDEFCLASS(DataInputWindow)
class DataInputWindow : public SFRPlainWindow {
    SFMSEALCOPY(DataInputWindow)
protected:
    SFREditboxControlPtr _textHour;             // Text control to input hours
    SFREditboxControlPtr _textMinute;           // Text control to input minutes
    SFREditboxControlPtr _textContent;          // Text control to input content
    SFRButtonControlPtr _buttonOk;              // For OK button
    SFRButtonControlPtr _buttonCancel;          // Cancel button
    SFRLabelControlPtr _labelDate;              // Date label
    SFRLabelControlPtr _labelHour;              // Hour label
    SFRLabelControlPtr _labelMinute;            // Minute label
    SInt16 _fontHeight;                         // Height of font
    SFXDateConst _date;                         // Date of today
public:
    DataInputWindow(SFXDateConstRef date) static_throws;
    virtual ~DataInputWindow(Void);
    virtual Void MakeData(SFXDateConstRef start, SFXAnsiStringConstRef summary) = 0;

    HANDLER_DECLARE_VOIDRENDER(OnRenderContent)         // Handler for draw
    HANDLER_DECLARE_BOOLEVENT(OnKey)                    // Handler for key
    HANDLER_DECLARE_VOIDCONTROL(OnOkButtonControl)      // Button handler
    HANDLER_DECLARE_VOIDCONTROL(OnCancelButtonControl)  // Button handler
	private:
	SFRResponderPtr next;
	HANDLER_DECLARE_VOIDCONTROL(OnEditboxHandler)
};

SFMTYPEDEFCLASS(CreateWindow)
class CreateWindow : public DataInputWindow {
    SFMSEALCOPY(CreateWindow)
public:
    CreateWindow(SFXDateConstRef date);
    virtual Void MakeData(SFXDateConstRef start, SFXAnsiStringConstRef summary);
};

SFMTYPEDEFCLASS(ModifyWindow)
class ModifyWindow : public DataInputWindow {
    SFMSEALCOPY(ModifyWindow)
private:
    VCalendarFilePtr _vcal;                 // Save data for modifying
public:
    ModifyWindow(SFXDateConstRef date, VCalendarFilePtr vcal);
    virtual Void MakeData(SFXDateConstRef start, SFXAnsiStringConstRef summary);
};

#endif //__DATAINPUTWINDOW_HPP
