#include "SyncScheduler.hpp"
#include "OneDayWindow.hpp"
#include "DataInputWindow.hpp"
#include "SoftkeyWindow.hpp"
#include "MainWindow.hpp"
#include "syncscheduler_res.h"
#define LABEL_SOFTKEY1          ("Finish")
#define LABEL_SOFTKEY2          ("Delete")        
#define LABEL_SOFTKEY3          ("Select")
#define MENU_TOP_MARGIN    20
#define MENU_SIDE_MARGIN   20
#define MENU_CURSOR_WIDTH  10

OneDayWindow::OneDayWindow(SFXDateConstRef date) : SFRPlainWindow(SFRApplication::GetInstance(), SyncScheduler::CalculateWindowPosition()), _date(date) static_throws
{
    SoftkeyWindowPtr softkey;
    // Registration of soft key
    SoftkeyWindow::Initialize(this);
    if (static_try()) {
        // Registration of drawing handler
        static_throw(RegisterHandler(SREVT_RESPONDER_RENDER, SRP16_RENDER_CONTENT,
        HANDLER_BEFORE, HANDLER_FUNCTION(OnRenderContent)));
        if (static_try()) {
            // Registration of key handler
            static_throw(RegisterHandler(SFEVT_KEY, HANDLER_AFTER, HANDLER_FUNCTION(OnKey)));
            if (static_try()) {
                //static_throw(RegisterHandler(static_cast<SFCEventEnum>(SoftkeyWindow::USEREVT_SOFTKEY), HANDLER_AFTER, HANDLER_FUNCTION(OnSoftkey)));
                if (static_try()) {
                    if ((softkey = SoftkeyWindow::GetInstance()) != null) {
                        static_throw(softkey->Register(this, SoftkeyWindow::SOFTKEY_1, LABEL_SOFTKEY1));
                        if (static_try()) {
                            static_throw(softkey->Register(this, SoftkeyWindow::SOFTKEY_2, LABEL_SOFTKEY2));
                            if (static_try()) {
                                static_throw(softkey->Register(this, SoftkeyWindow::SOFTKEY_3, LABEL_SOFTKEY3));
                                if (static_try()) {
                                    SetReference(softkey->Bind(this));
                                    VCalCollectionPtr vcalCollection = SyncScheduler::GetVCalCollection();
                                    _cursor = 0;
                                    _maxline = GetContentWorld().GetHeight() / SFXGraphics::GetFontHeight(AEE_FONT_NORMAL);
                                    _displine = 0;
                                    _index = vcalCollection->GetIndex(date.GetYear(), date.GetMonth(), date.GetDay());
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    return;
}


OneDayWindow::~OneDayWindow(Void)
{
    SoftkeyWindow::Terminate(this);
    return;
}


// Drawing handler
HANDLER_IMPLEMENT_VOIDRENDER(OneDayWindow, OnRenderContent, graphics)
{ 
	// Get screen size
    SFXRectangle rect(graphics->GetDeviceRectangle());
    SFXAnsiString title;
    SIntN i;
    SInt32 offset;

    graphics->FillRectangle(rect, SFXRGBColor(255, 255, 255, 0));
    graphics->SetForeColor(SFXRGBColor(0, 0, 0, 0));
    rect.Deflate(MENU_SIDE_MARGIN, MENU_TOP_MARGIN);
    SInt16 height = graphics->GetFontHeight();
    rect.SetHeight(height);
    SInt32 line = 0;
    if (_displine == 0) {
        graphics->DrawText("NEW", rect, IDF_ALIGN_LEFT);
        rect.AddY(height);
        ++line;
        offset = 0;
    }
    else {
        offset = _displine - 1;
    }
    SFXList<VCalendarFilePtr>::Enumerator e = SyncScheduler::GetVCalCollection()->GetEnumerator(_index + offset);
	VCalCollectionPtr vcalCollection = SyncScheduler::GetVCalCollection();      
	_number = vcalCollection->GetNumberOfVCal(_index, _date.GetYear(), _date.GetMonth(), _date.GetDay());
    for (i = offset; i < _number && line <= _maxline; ++i) {
        VCalendarPtr vcal = e.GetNext();
        title = SFXAnsiString::Format("%02d:%02d ", vcal->GetStartDate().GetHour(),
        vcal->GetStartDate().GetMinute());
        title += vcal->GetSummary();
        graphics->DrawText(title, rect, IDF_ALIGN_LEFT);
        rect.AddY(height);
        ++line;
    }
    // Triangular cursor
    SFXTriangle triangle(0, 0, 0, height, MENU_CURSOR_WIDTH, height / 2);
    triangle.Offset(MENU_SIDE_MARGIN - MENU_CURSOR_WIDTH, MENU_TOP_MARGIN);
    // move to the line indicated by current position. 
    triangle.Offset(0, _cursor * height); 
    // The fill color is white. 
    graphics->SetFillColor(SFXRGBColor(255, 255, 255, 0)); 
    graphics->DrawTriangle(triangle);
    return;
}


// Key handleriwhen the window is in the foreground@and the key is pushed, it is executed. j
HANDLER_IMPLEMENT_BOOLEVENT(OneDayWindow, OnKey, event)
{
    Bool result(false);
    switch (event.GetP16()) {
    case AVK_CLR:               
        // Clear key is pressed
        // Close window
        result = Invoke(SFXEvent(SREVT_RESPONDER_TERMINATE,SRP16_TERMINATE_INVOKE, true));
        break;
    case AVK_SOFT2:		
        // If softkey 2 is pressed
        if (_cursor > 0) {
            SFRMultiDialog::ParamRec param;
            param.text = "Are you sure you want to delete this item?";
            param.ok = "Yes";
            param.cancel = "No";
            param.focus = SFRMultiDialog::FOCUS_CANCEL;
            param.image = SFBShell::GetInstance()->LoadResImage(SYNCSCHEDULER_RES_FILE, ACCEPT);
            if ((dialog = ::new SFRMultiDialog(SyncScheduler::CalculateDialogPosition(SFRMultiDialog::GetMargin(), 130), "Confirmation", param)) != null) {
                SyncScheduler::OpenStandardDialog(dialog, HANDLER_FUNCTION(OnDelete));
            }
        } 
        break;
    case AVK_SELECT:			
        // Select key is pushed
        if (_cursor <= 0) {
            // Create new window
            ::new CreateWindow(_date);
        } 
        else {
            // Data that cursor points to
            VCalendarFilePtr vcal =
            SyncScheduler::GetVCalCollection()->Get(
            _index + _displine + _cursor - 1);
            // Close window
            ::new ModifyWindow(_date, vcal);
        }
        result = true;
        break;
    case AVK_DOWN:
        // If cursor is not at the bottom of data
        if (_cursor + _displine < _number) {
            // If the cursor is not at the bottom of the screen. 
            if (_cursor >= _maxline - 1) {
                // Move displayed line down
                ++_displine; 
            } 
            else {
                // Move cursor down
                ++_cursor; 
            }
        }
        else {
            // If cursor is at the bottom of screen
            // move cursor at the top of screen
            _displine = 0;
            _cursor = 0;
        }
        InvalidateContent(); // A new area is registered
        result = true;
        break;
    case AVK_UP:
        // if the cursor is at the top of screen
        if (_cursor == 0) {
            // if line displayed can be moved up
            if (_displine > 0) {
                --_displine; // Move the line up
            }
            else {
                // if line displayed cannot be moved up
                // move cursor at the bottom of screen
                _cursor = (_number < _maxline - 1) ?
                static_cast<SInt16>(_number) : _maxline - 1;
                _displine = (_number > _maxline - 1) ?
                (static_cast<SInt16>(_number) - _maxline + 1) : 0;
            }
        } 
        else {
            --_cursor;  // Move cursor up
        }
        InvalidateContent(); // A new area is registered
        result = true;
        break;
    case AVK_SOFT1:		
        // If softkey1 is pressed
        #define     MULTIDIALOG_HEIGHT      (136)
        enum 
		{
            OPERATION_PREFERENCE,
            OPERATION_VERSION,
            OPERATION_LIMIT
        };
        SFRMultiDialogPtr dialog;
        SFRMultiDialog::ParamRec param;
        SFCError error(SFERR_NO_ERROR);
        param.text = "Are You Done?";
        param.ok = "Yes";
        param.cancel = "No";
        param.focus = SFRMultiDialog::FOCUS_CANCEL;
        param.image = SFBShell::GetInstance()->LoadResImage(SYNCSCHEDULER_RES_FILE, ACCEPT);
        if ((dialog = ::new SFRMultiDialog(SyncScheduler::CalculateDialogPosition(SFRMultiDialog::GetMargin(), MULTIDIALOG_HEIGHT), "Confirmation", param)) != null) {
            SyncScheduler::OpenStandardDialog(dialog, HANDLER_FUNCTION(FromSoftkey1));
        } 
        break;
    }
    return result;
}


HANDLER_IMPLEMENT_VOIDDIALOG(OneDayWindow, FromSoftkey1, result, dialog)
{
    switch (result) {
    case SRP16_OK:
        SyncScheduler::CloseStandardDialog(dialog);
        SFRApplication::Terminate();
        break;
    case SRP16_ESCAPE:
    case SRP16_CANCEL:
        SyncScheduler::CloseStandardDialog(dialog);
        break;
    }
    return;
}


HANDLER_IMPLEMENT_VOIDDIALOG(OneDayWindow, OnDelete, result, dialog)
{
    VCalendarFilePtr vcal =  SyncScheduler::GetVCalCollection()->Get(_index + _displine + _cursor -1);
    switch (result) {
    case SRP16_OK:
        // Close the comfirm window
		SyncScheduler::CloseStandardDialog(dialog);
		InvalidateContent();
        // delete
        SyncScheduler::GetVCalCollection()->Remove(vcal);
        _cursor--;
        break;
    case SRP16_ESCAPE:
    case SRP16_CANCEL:
        SyncScheduler::CloseStandardDialog(dialog);
        break;
    }

}

