#ifndef __VCALENDAR_HPP
#define __VCALENDAR_HPP

#include <SophiaFramework.hpp>

SFMTYPEDEFCLASS(VCalendar)
SFMTYPEDEFCLASS(VCalendarFile)
SFMTYPEDEFCLASS(VCalCollection)

class VCalendar {
private:
    SFXAnsiString _version;			// For VERSION:
    SFXDate _startDate;				// For DSTART:
    SFXDate _endDate;				// For DTEND:
    SFXDate _lastModifiedDate;		// For LAST-MODIFIED:
    SFXAnsiString _summary;			// For SUMMARY:
    SFXAnsiString _others;			// For OTHERS:
public:
    VCalendar(Void);				// Constructors
    VCalendar(SFXAnsiStringConstRef string);
    VCalendar(SFXAnsiStringConstRef version, SFXDateConstRef startDate,
        SFXDateConstRef endDate, SFXDateConstRef lastModifiedDate,
        SFXAnsiStringConstRef summary);
    Void Set(SFXDateConstRef startDate, SFXDateConstRef endDate,			// Set VCalendar
        SFXDateConstRef lastModifiedDate, SFXAnsiStringConstRef summary);
    Void SetVersion(SFXAnsiStringConstRef version);							// Set VCalendar version
    SFXDate GetStartDate(Void) const;										// Returns the start date
    SFXDate GetEndDate(Void) const;											// Returns the end date
    SFXDate GetLastModifiedDate(Void) const;								// Returns last modified date
    SFXAnsiString GetSummary(Void) const;									// Returns the summary
    Void Import(SFXAnsiStringConstRef string);								// Import new data
    Void Export(SFXAnsiStringPtr string) const;								// Export new data

    static SFXAnsiString PublishXid(SFXDateConstRef currentDT);
};


class VCalendarFile : public VCalendar {
private:
    SFXPath _fileName;              // file name
    SFXAnsiString _flag;            // new make : "n" , modify:  "m", delete: "d", nothing: "-"
public:
    VCalendarFile(SFXPathConstRef fileName, SFXAnsiStringConstRef flag);	// Constructor
    VCalendarFile(SFXAnsiStringConstRef content,
        SFXPathConstRef fileName, SFXAnsiStringConstRef flag);
    VCalendarFile(SFXAnsiStringConstRef version, SFXDateConstRef startDate,
        SFXDateConstRef endDate, SFXAnsiStringConstRef summary);
    VCalendarFile();
    Void SetFileName(SFXPathConstRef fileName);		// Sets file name
    Void SetFlag(SFXAnsiStringConstRef flag);		// Sets the flag
    SFXPath GetFileName(Void) const;				// Returns the file name
    SFXAnsiString GetFlag(Void) const;				// Returns flag status
    SFCError SaveToFile(Void) const;				// Write onto file
    SFCError LoadFromFile(Void);					// Read from file
};


class VCalCollection {
private:
    // Manage VCalendarFile variables
    // This list should be always ordered by date ( _startDT )
    SFXList<VCalendarFilePtr> _list;
    SFXAnsiString _lastSyncTime;        // The last synchronized time with server
public:
    VCalCollection(Void);
    ~VCalCollection(Void);
    Void SetLastSyncTime(SFXAnsiStringConstRef lastSyncTime);
    SFXAnsiString GetLastSyncTime(Void) const;
    Void Append(VCalendarFilePtr vcalendar);
    Void Update(VCalendarFilePtr vcal);
    Void Delete(VCalendarFilePtr vcal);
    Void Remove(VCalendarFilePtr vcal);
    VCalendarFilePtr Get(SInt32 index) const;
    VCalendarFilePtr Get(SFXPathConstRef fileName) const;
    SFXList<VCalendarFilePtr>::Enumerator GetEnumerator(SInt32 index) const;
    SInt32 GetIndex(SInt16 year, SInt16 month, SInt16 day) const;
    SInt32 GetNumberOfVCal(SInt32 today, SInt16 year, SInt16 month, SInt16 day) const;
    Bool IsDataExist(SInt16 year, SInt16 month, SInt16 day) const;
    SFCError ReadIndexFile(Void);			// Read from file
    SFCError WriteIndexFile(Void);			// Write to file
    SFXAnsiString MakeListForSync(Void);
};

#endif // __VCALENDAR_HPP //
